# Package this module.
package ImunifyHook;

# Deployment instructions are here:
# https://documentation.cpanel.net/display/DD/Guide+to+Standardized+Hooks+-+Hook+Action+Code#fcc4f1c5879a444aaa81888aff4fe4f4

# This file is used by Imunify360 to process events such as package change
# It should be located at /usr/local/cpanel/ImunifyHook.pm


# Return errors if Perl experiences problems.
use strict;
use warnings;

# Properly decode JSON.
use JSON;
# Use unix timestamp with milliseconds
use Time::HiRes qw/time/;

BEGIN { unshift @INC, '/usr/local/cpanel'; }

# Embed hook attributes alongside the action code.
sub describe {
    my $create = {
        'category' => 'Whostmgr',
        'event'    => 'Accounts::Create',
        'stage'    => 'post',
        'hook'     => 'ImunifyHook::hook_processing',
        'exectype' => 'module',
    };

    my $change_package = {
        'category' => 'Whostmgr',
        'event'    => 'Accounts::change_package',
        'stage'    => 'post',
        'hook'     => 'ImunifyHook::hook_processing',
        'exectype' => 'module',
    };

    my $modify = {
        'category' => 'Whostmgr',
        'event'    => 'Accounts::Modify',
        'stage'    => 'post',
        'hook'     => 'ImunifyHook::hook_processing',
        'exectype' => 'module',
    };

    my $delete = {
        'category' => 'Whostmgr',
        'event'    => 'Accounts::Remove',
        'stage'    => 'post',
        'hook'     => 'ImunifyHook::hook_processing',
        'exectype' => 'module',
    };

    return [ $create, $change_package, $modify, $delete ];
}

sub print_error {
    my ($hook_name, $msg) = @_;
    print STDERR "ImunifyHook:$hook_name: $msg\n";
}

sub write_data {

    my ($hook_name, $data) = @_;
    my $user;
    if (exists $data->{newuser}) {
        $user = $data->{newuser};
    } else {
        $user = $data->{user};
    }
    my $timestamp = time();
    my $result = 0;
    my $file_name = "/var/imunify360/hooks/$user.$hook_name.$timestamp";

    my $fr = open(my $fh, '>', "$file_name.json.tmp");
    unless ($fr) {
        print_error($hook_name, "Unable to open $file_name.json.tmp to save data for Imunify\n");
        return 0;
    }
    $result = print $fh encode_json($data);
    unless ($result) {
        print_error($hook_name, "Unable to write to $file_name.json.tmp\n");
    } else {
        $result = close $fh;
        unless ($result) {
            print_error($hook_name, "Unable to ends write to $file_name.json.tmp\n");
        } else {
            $result = rename "$file_name.json.tmp", "$file_name.json";
        }
    }

    unless ($result) {
        unlink "$file_name.json.tmp"
    }

    return $result;

}

sub hook_processing {
    # Get the data that the system passes to the hook.
    my ( $context, $data ) = @_;

    # Set success and failure messages.
    my $result  = 0;  # This boolean value is set to fail.
    my $message = "Applying Imunify settings";

    # write $data to temporary file
    my ($module, $method) = split(/\s*::\s*/, $context->{event});
    $result = write_data($method, $data);

    unless ($result)
    {
        $message = 'Unable to apply Imunify settings';
    }

    # Return the hook's result and message.
    return $result, $message;
}

1;
